#!/usr/bin/perl

# Copyright (c) 2002 Oregon State University - Network Engineering
# All rights reserved.

# $Id: Maintain.pm,v 1.8 2003/12/18 17:03:48 vollmerk Exp $
# $Source: /a1/cvsroot/maint/staging/Maintain.pm,v $

# This file contains the functions that can be used to update
#   the staging database for the Maintain toolkit.
#
# To enable this library for your application you can either set
#   the environment variable:
#
# export PERL5LIB=/some/directory
#
# Or you assuming you put this file in /path/to/this/file you can 
#   set the following in your script:
#
# use lib qw(/path/to/this/file);
#
# use Maintain;
#
# $maintain = new Maintain('username', 'password');
#
# @array_of_hosts = $maintain->get_all_hosts();
#

package Maintain;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %maintain);

use DBI;
use Data::Dumper;
# for use in converting the IP addresses into integers
use Socket;

# This is the Maintain constructor and is used to create
#   a connection to the database.  You pass this your username
#   and password which you receive from your administrator
#
sub new {
        my ($class, $user, $password) = @_;

        my $database = 'maintain'; 

        my $self = 
                {
                        _username  => $user,
                        _database  => $database,
                        _sth_cache => {},
                        _domains   => {},
                        _connect  => {
                                dbd      => 'mysql',
                                host     => 'localhost',
                                port     => '3306',
                                username => $user,
                                password => $password 
                        },
                _dbh       => '',
                _debug  => 1,
        };

        $Maintain::maintain{$user} = bless ($self, $class);

        $self->{_dbh} = $self->dbh( $database );

        return $self;
} # new


# Get a database handle for the specified database
#
sub dbh {
  my ($self, $database) = @_;

  my $dbh = '';

  if ( $self->{_dbh} ) {
    return $self->{_dbh};
  }
  else {
    my $connect_string = [ sprintf("dbi:%s:database=%s;host=%s;port=%s", 
                   $self->{_connect}{dbd}, 
                   $self->{_database},
                   $self->{_connect}{host},
                   $self->{_connect}{port}),
                   $self->{_connect}{username},
                   $self->{_connect}{password} ];

    $dbh = DBI->connect( @{$connect_string}, 
		      {PrintError => 1,
		       RaiseError => 1,
		       AutoCommit => 1});
  }

  if ( $database ) {
    $dbh->do("use $database") || die $dbh->errstr;
  }

  return $dbh;
} # dbh


# This subroutine will update a host with the following values:
#
#   ip = IP address in 0.0.0.0 format
#   name = Hostname (anything before the domain)
#   domain = The fully-qualified domain name (rcn.oregonstate.edu)
#   ha = Hardware address which is a 12 character hexidecimal
#   type = Hardware type (optional)
#   os = Operating System (optional)
#   location = Location of this computer (optional)
#   serial = Serial number (optional)
#   other_id = Some other ID you might want to categorize here (optional)
#
# Usage:
#
# $maintain->update_host('128.193.43.120', 'somehost', 'lljk.oregonstate.edu', '012345678901',
# 			'Intel P4', 'Windows XP', 'Weniger 693', 'xyz', 'Dell service tag?');
#
sub update_host {
	my ($self, $ip, $name, $domain, $ha, $type, $os, $location, $serial, $other_id, $action) = @_;

	# check to see that they are running the proper version
	if ( not $action ) {
		print "Sorry, you need to upgrade your script to use the new subroutines.\n";
		return;
	}

	# check the IP and then convert it to integer
	if ( $ip == 0 ) {
		# this is a dynamic host, leave it alone
	}
	elsif ( $ip =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/ ) {
		$ip = unpack N => inet_aton($ip);
	}
	else {
		$self->show_error("$ip is not a valid IP address.");
		return;
	}

	# check the name and domain at the same time
	if ($name !~ /^[0-9A-Za-z\-]+$/ or $domain !~ /^[0-9A-Za-z\-\.]+$/ ) {
		$self->show_error("$name.$domain is not a valid hostname.  Should only contain (0-9A-Za-z-.).");		
		return;
	}

	# check the ha for 12 chars and hexidecimal
	if ( $ha !~ /[0-9a-fA-F]{12,12}/ ) {
		$self->show_error("$ha is an invalid hardware address.  Use only hexidecimal numbers (0-9 and A-F).");
		return;
	}

	# No warnings so let's insert this bugger!
	#   The sth_cache allows us to cache this statement handle so we don't have
	#   to rebuild it over and over again.
	if ( ! $self->{_sth_cache}{update_host_insert} ) {
		my $sql = qq{INSERT INTO staging (ip,name,domain,ha,type,os,location,serial,other_id,username,action)
				VALUES (?,?,?,?,?,?,?,?,?,?,?)};
		$self->{_sth_cache}{update_host_insert} = $self->{_dbh}->prepare($sql);
	}

	# we have an statement handle so let's get to it
	my $sth = $self->{_sth_cache}{update_host_insert};

	print "$ip, $name, $domain, $ha, $type, $os, $location, $serial, $other_id, $self->{_username}, $action\n" if $self->{_debug}; 

	$sth->execute($ip, $name, $domain, $ha, $type, $os, $location, $serial, $other_id, $self->{_username}, $action);

} # update_host


# This is one of the public subs for updating a host.  This one will be keyed on
#   the MAC address.
#
sub update_host_ha {
	my ($self, $ip, $name, $domain, $ha, $type, $os, $location, $serial, $other_id) = @_;

	$self->update_host($ip, $name, $domain, $ha, $type, $os, $location, $serial, $other_id, 'update');

} # update_host_ha


# The other public sub for updating hosts.  This will be keyed on the serial number of
#   the host.
#
sub update_host_serial {
	my ($self, $ip, $name, $domain, $ha, $type, $os, $location, $serial, $other_id) = @_;

	$self->update_host($ip, $name, $domain, $ha, $type, $os, $location, $serial, $other_id, 'update_serial');

} # update_host_serial


# Given a only a hardware address you can delete a host assuming you
#   have permissions to do so.
#
#   Usage: 
#
#   $maintain->delete_host('02ae236fde34');
#
sub delete_host {
	my ($self, $ha) = @_;

	print "ha: $ha\n";

	# check the ha for 12 chars and hexidecimal
	if ( $ha !~ /[0-9a-fA-F]{12,12}/ ) {
		$self->show_error("$ha is an invalid hardware address.  Use only hexidecimal numbers (0-9 and A-F).");
	}

	# any errors above and print a message saying it was not updated
	
	# No warnings so let's insert this bugger!
	#   The sth_cache allows us to cache this statement handle so we don't have
	#   to rebuild it over and over again.
	if ( ! $self->{_sth_cache}{delete_host_insert} ) {
		my $sql = qq{INSERT INTO staging (ha,username,action) VALUES (?,?,?)};
		$self->{_sth_cache}{delete_host_insert} = $self->{_dbh}->prepare($sql);
	}

	# we have an statement handle so let's get to it
	my $sth = $self->{_sth_cache}{delete_host_insert};

	$sth->execute($ha, $self->{_username}, 'delete');
} # delete_host

# Sometimes you may not want to delete by hardware address because you have
#   the same hardware address in many different subnets, workgroups, etc.
#   Use this subroutine to delete a host based on its IP.
#
#   Usage: 
#
#   $maintain->delete_host_by_ip('128.193.136.33');
#
sub delete_host_by_ip  {
        my ($self, $ip) = @_;

        # check the ha for 12 chars and hexidecimal
        if ( $ip !~ /\d+\.\d+\.\d+\.\d+/ ) {
        	$self->show_error("$ip is an invalid IP address.  Must be of the format x.x.x.x.");
        }
	else {
                $ip = unpack N => inet_aton($ip);
	}

        # any errors above and print a message saying it was not updated
        # No warnings so let's insert this bugger!
        #   The sth_cache allows us to cache this statement handle so we don't have
        #   to rebuild it over and over again.
        if ( ! $self->{_sth_cache}{delete_host_by_ip_insert} ) {
	        my $sql = qq{INSERT INTO staging (ip,username,action) VALUES (?,?,?)};
		$self->{_sth_cache}{delete_host_by_ip_insert} = $self->{_dbh}->prepare($sql);
        }

        # we have an statement handle so let's get to it
        my $sth = $self->{_sth_cache}{delete_host_by_ip_insert};
        $sth->execute($ip, $self->{_username}, 'delete');

} # delete_host_by_ip


# This allows you to delete a host based on its hardware address.
#
#   Usage: 
#
#   $maintain->delete_host_by_serial('<arbitrary>');
#
sub delete_host_by_serial {
        my ($self, $serial) = @_;

        if ( not $serial ) {
        	$self->show_error("Please enter some serial number.");
		return;
        }

        # any errors above and print a message saying it was not updated
        # No warnings so let's insert this bugger!
        #   The sth_cache allows us to cache this statement handle so we don't have
        #   to rebuild it over and over again.
        if ( ! $self->{_sth_cache}{delete_host_by_serial_insert} ) {
	        my $sql = qq{INSERT INTO staging (serial,username,action) VALUES (?,?,?)};
		$self->{_sth_cache}{delete_host_by_serial_insert} = $self->{_dbh}->prepare($sql);
        }

        # we have an statement handle so let's get to it
        my $sth = $self->{_sth_cache}{delete_host_by_serial_insert};
        $sth->execute($serial, $self->{_username}, 'delete');

} # delete_host_by_serial


# This will return all of the hosts configured for your current zone
#   It returns an array of hash references.  To use do the following:
#
#   Usage:
#
#   my @hosts = $maintain->get_all_hosts();
#
#   foreach my $host (@hosts) {
#	print "$host->{ip} is $host->{name}.$host->{domain}\n";
#	here you can do whatever you need to do with the host
#   }
#
#   The fields contained in the host hash reference are as follows:
#
#   ip = IP address in 0.0.0.0 format
#   name = Hostname (anything before the domain)
#   domain = The fully-qualified domain name (rcn.oregonstate.edu)
#   ha = Hardware address which is a 12 character hexidecimal 
#   type = Hardware type (optional)
#   os = Operating System (optional)
#   location = Location of this computer (optional)
#   serial = Serial number (optional)
#   other_id = Some other ID you might want to categorize here (optional)
#
sub get_all_hosts {
	my ($self) = @_;

	my @hosts = ();

	# select * from staging_permissions where username=$self->{_username}
	my $sql = qq{SELECT * FROM staging_permissions WHERE username=?};
	my $perm_sth = $self->{_dbh}->prepare($sql);
	$perm_sth->execute($self->{_username});

	# select * from host where zone=$zone_id
	$sql = qq{SELECT * FROM host WHERE zone=?};
	my $host_sth = $self->{_dbh}->prepare($sql);

	while ( my $perms = $perm_sth->fetchrow_hashref() ) {

	$host_sth->execute($perms->{zone});

	# we need to associate the domain with a real name and 
	#   not a domain id
	my %domains    = ();
	my $domain_sql = qq{SELECT * FROM domain WHERE id=?};
	my $domain_sth = $self->{_dbh}->prepare($domain_sql);

#	my $ip_sql = qq{SELECT inet_ntoa(address) FROM ip WHERE id=?};
#	my $ip_sth = $self->{_dbh}->prepare($ip_sql);

	# return array of hashrefs
	while ( my $host = $host_sth->fetchrow_hashref() ) {

		# convert the decimal ip to 0.0.0.0 notation
		$host->{ip} = inet_ntoa(pack N => $host->{ip});

		# set the domain name accordingly
		#   This bit of code will save on domain lookups
		if ( $domains{$host->{domain}} ) {
			$host->{domain} = $domains{$host->{domain}};
		}
		else {
			$domain_sth->execute($host->{domain});
			my $dom = $domain_sth->fetchrow_hashref();
			$domains{$dom->{id}} = $dom->{name};

			$host->{domain} = $dom->{name};
		}

		push(@hosts, $host);
	}

	}

	return @hosts;

} # get_all_hosts


# Just show an error for a given problem
#
sub show_error {
	my ($self, $error) = @_;

	print "No update: $error\n";
} # show_error

1;
